/*
* pixelserv.c a small mod to public domain
** server.c -- a stream socket server demo
* from http://beej.us/guide/bgnet/
* single pixel http string from
* http://proxytunnel.sourceforge.net/pixelserv.php
* V1 Proof of concept mstombs www.linkysinfo.org 06/09/09
* V2 usleep after send to delay socket close 08/09/09
* V3 TCP_NODELAY not usleep 09/09/09
* V4 daemonize with syslog 10/09/09
* V5 usleep back in 10/09/09
* V6 only use IPV4, add linger and shutdown to avoid need for sleep 11/09/09
* Consistent exit codes and version stamp
* V7 use shutdown/read/shutdown to cleanly flush and close connection
* V8 add inetd and listening IP option
* V9 minimalize
*/
#define VERSION "V9a"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <syslog.h>

#define PORT "80"  // the port users will be connecting to

#define BACKLOG 20 // how many pending connections queue will hold

int main( int argc, char * argv[] )
{
	int sockfd, new_fd;  // listen on sock_fd, new connection on new_fd
	struct addrinfo hints, *servinfo;
	struct sockaddr_storage their_addr; // connector's address information
	socklen_t sin_size;
	int yes = 1;
	int rv;
	char buf[128];
        char ip_addr[INET_ADDRSTRLEN] = "";
	int use_ip = 0;

/*	static const char http_head[] =
		"HTTP/1.1 200 OK\r\n"
		"Content-type: image/gif\r\n"
		"Accept-ranges: bytes\r\n"
		"Content-length: 43\r\n"
		"\r\n"; */
/* converted to numeric with */
/*	int i;
	for ( i = 0; i < sizeof http_head; i++)
	{
		printf("%u,", (int) http_head[i]);
	}
*/
	static const unsigned char httpnullpixel[] = {
		72,84,84,80,47,49,46,49,32,50,48,48,32,79,75,13,10,67,111,110,
		116,101,110,116,45,116,121,112,101,58,32,105,109,97,103,101,
		47,103,105,102,13,10,65,99,99,101,112,116,45,114,97,110,103,
		101,115,58,32,98,121,116,101,115,13,10,67,111,110,116,101,
		110,116,45,108,101,110,103,116,104,58,32,52,51,13,10,13,10,
/*		};
	static const unsigned char pixel[] = { */
		  71, 73, 70, 56, 57, 97, 1, 0, 1, 0, 128, 
		  0, 0, 255, 255, 255, 0, 0, 0, 33, 249, 4,
		  1, 0, 0, 0, 0, 44, 0, 0, 0, 0, 1, 0, 1, 0,
		  0, 2, 2, 68, 1, 0, 59 };

	if ( argc > 1 ) {
	/* first check if "-i" option is used */
		if (0 == strcmp(argv[1], "-i"))	{
			fwrite ( httpnullpixel, 1, sizeof httpnullpixel, stdout );
			/* flush read buffer ? */
			exit ( EXIT_SUCCESS );
		}

	/* assume an IP is defined by first arg */
		strncpy (ip_addr, argv[1], INET_ADDRSTRLEN);
		ip_addr[INET_ADDRSTRLEN - 1] = '\0';
		use_ip = 1;
	}
			
	printf( "%s %s compiled: %s from %s\n", argv[0], VERSION, __DATE__ " " __TIME__  , __FILE__ );

	if(daemon(0, 0)<0) {
		syslog(LOG_ERR, "failed to daemonize, quitting: %m");
		exit( EXIT_FAILURE );
	}

	openlog("pixerlserv", LOG_PID|LOG_CONS, LOG_DAEMON);

	memset(&hints, 0, sizeof hints);
	hints.ai_family = AF_INET; //AF_UNSPEC; /* AF_INET restricts to IPV4 */
	hints.ai_socktype = SOCK_STREAM;
	if ( use_ip == 0 ) {
		hints.ai_flags = AI_PASSIVE; // use my IP
	}

	if ((rv = getaddrinfo( (use_ip == 1)? ip_addr: NULL, PORT, &hints, &servinfo)) != 0) {
		syslog(LOG_ERR, "getaddrinfo: %s", gai_strerror(rv));
		exit( EXIT_FAILURE );
	}

	if ( ((sockfd = socket(servinfo->ai_family, servinfo->ai_socktype, servinfo->ai_protocol)) < 0)
	    ||
	     (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) < 0) 
	    ||
	     (bind(sockfd, servinfo->ai_addr, servinfo->ai_addrlen) < 0)
	    ||
	     (listen(sockfd, BACKLOG) < 0) ) {
		syslog(LOG_ERR, "Abort: %m");
		exit( EXIT_FAILURE );
	}

	freeaddrinfo(servinfo); // all done with this structure

	syslog(LOG_NOTICE, "waiting for connections on port " PORT);

	while(1) {  // main accept() loop
		sin_size = sizeof their_addr;
		new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size);
		if (new_fd < 0) {
			syslog(LOG_WARNING, "accept: %m");
			continue;
		}

		rv = send(new_fd, httpnullpixel, sizeof httpnullpixel, 0);
		if (rv < 0) {
			syslog(LOG_WARNING, "send: %m");
		}

		/* clean way to flush read buffers and close connection */
    		if (shutdown(new_fd, SHUT_WR) == 0) {
          		while( read(new_fd, buf, sizeof buf) > 0 );
		}

    		shutdown(new_fd, SHUT_RD);
		close(new_fd); 
	}

	return EXIT_SUCCESS;
}

